fit.mixture.link.mom <- function(y, m, J, extra.tx = null.tx, var.names = NULL)
{
	n <- length(y)
	qq <- 2+J
	if (length(m) == 1) m <- rep(m,n)

	# This is currently not used - it's a MOM estimator for kappa,
	# based on the 2nd factorial moment of the outcome
	kappa.hat <- function(p, Pi, y, m)
	{
		if (length(m) == 1) m <- rep(m,n)
		V <- find.vertices(p, Pi)
		k <- ncol(V)
		a <- diag(V %*% t(V))
		b <- rowMeans(V)

		A <- k * mean( y*(y-1) / (m*(m-1)) )
		B <- sum(Pi * a)
		C <- sum(Pi * (k*b)^2)
		
		printf("A = %f, B = %f, C = %f\n", A, B, C)
		return( (B - k*A) / (k*A - C) )
	}

	var.p.hat <- function(p, Pi, kappa, m, n)
	{
		V <- find.vertices(p, Pi)
		k <- ncol(V)
		a <- diag(V %*% t(V))
		b <- rowMeans(V)

		1/n^2 * sum(p * (1 - m*p) / m) +
			1/n^2 * sum((m-1)/m) * sum(Pi * (a + kappa * (k*b)^2) / (k*(1 + kappa*k)))
	}
	
	# Estimate for p using sample proportions
	p.hat <- mean(y/m)

	# Quick estimate for Pi based on maximizing usual finite mixture
	fit.binmix.out <- fit.binmix.mle(y, m, J)
	Pi.hat <- sort(fit.binmix.out$theta.hat$Pi)

	# Quick estimate for kappa based on maximizing profile likelihood
	if (J == 2) {
		res <- fit.mixture.link.kappa.only.mle(y, m, J, p.hat, Pi.hat)		
	} else {
		res <- fit.mixture.link.kappa.only.mle(y, m, J, p.hat, Pi.hat, method = "betaapprox")
	}
	kappa.hat <- res$theta.hat$kappa
	
	V <- var.p.hat(p.hat, Pi.hat, kappa.hat, m, n)
	
	# Note: follow SAS NLMIXED, treat Est/SE as t with df = n
	theta.hat <- c(p.hat, Pi.hat, kappa.hat)
	xi.hat <- NULL
	
	df <- n
	se <- sqrt(V)
	t.val <- p.hat / se
	p.val <- 2 * (1 - pt(abs(t.val), df = df))
	
	estimates <- cbind(p.hat, se, t.val, p.val)
	colnames(estimates) <- c("Estimate", "SE", "t-val", "P(|t|>t-val)")

	res <- list(estimates = estimates, vcov = V, df = df, qq = qq,
		description = "<Default>", theta.hat = theta.hat, xi.hat = xi.hat)
	return(res)
}

fit.mixture.link.x.mom <- function(y, m, X, J, extra.tx = null.tx, var.names = NULL, kappa.max = 1000)
{
	n <- length(y)
	d <- ncol(X)
	qq <- d+J+1
	if (length(m) == 1) m <- rep(m,n)
	
	# Compute the covariance matrix of (y[1]/m[1], ..., y[n]/m[n])
	# It's a diagonal matrix, so just return a vector with the diagonal
	var.p.hat <- function(X, Beta, Pi, kappa, m, n)
	{
		if (length(m) == 1) m <- rep(m,n)
		V.p <- numeric(n)
		p <- plogis(X %*% Beta)

		for (i in 1:n)
		{
			V <- find.vertices.x(X[i,], Beta, Pi)
			k <- ncol(V)
			a <- diag(V %*% t(V))
			b <- rowMeans(V)
			
			V.p[i] <- p[i] * (1 - m[i]*p[i]) / m[i] +
				(m[i]-1)/m[i] * sum(Pi * (a + kappa * (k*b)^2) / (k*(1 + kappa*k)))
		}

		return(V.p)
	}

	# Estimate for Beta based on least squares
	obj.fun <- function(Beta) { sum( ( y/m - plogis(X %*% Beta) )^2 ) }
	res <- optim(par = c(0,0,0), fn = obj.fun, method = "L-BFGS-B")
	Beta.hat <- res$par
	
	# Quick estimate for Pi based on maximizing usual finite mixture of
	# logistic regressions
	X.g <- list()
	for (j in 1:J) {
		X.g[[j]] <- X
	}
	res <- fit.binmix.x.mle(y, m, X.g)
	Pi.hat <- sort(res$theta.hat$Pi)
	
	# Quick estimate for kappa based on maximizing profile likelihood
	if (J == 2) {
		method <- "imhof"
	} else {
		method <- "betaapprox"
	}
	p.hat <- plogis(X %*% Beta.hat)

	# res <- fit.mixture.link.kappa.only.mle(y, m, J, p.hat, Pi.hat, method = method)
	# kappa.hat <- res$theta.hat$kappa

	loglik <- function(kappa)
	{
		ll <- numeric(n)
		for (i in 1:n)
		{
			ll[i] <- d.mixture.link.one(y[i], m[i], p.hat[i], Pi.hat,
				kappa, method = method, log = TRUE)
		}
		printf("kappa = %f, loglik = %f\n", kappa, sum(ll))
		return(sum(ll))
	}
	
	res <- optimize(loglik, interval = c(0,kappa.max),  maximum = TRUE)
	kappa.hat <- res$maximum
	
	# Estimate the variance of Beta.hat
	D.vec <- dlogis(X %*% Beta.hat)
	J <- -D.vec %*% t(rep(1,d)) * X
	A <- solve(t(J) %*% J) %*% t(J)
	V.p <- var.p.hat(X, Beta.hat, Pi.hat, kappa.hat, m, n)
	V.Beta <- A %*% diag(V.p) %*% t(A)
	

	# Note: follow SAS NLMIXED, treat Est/SE as t with df = n
	theta.hat <- c(Beta.hat, Pi.hat, kappa.hat)
	xi.hat <- NULL
	
	df <- n
	se.Beta <- sqrt(diag(V.Beta))
	t.val <- Beta.hat / se.Beta
	p.val <- 2 * (1 - pt(abs(t.val), df = df))
	
	estimates <- cbind(Beta.hat, se.Beta, t.val, p.val)
	colnames(estimates) <- c("Estimate", "SE", "t-val", "P(|t|>t-val)")
	
	res <- list(estimates = estimates, vcov = V.Beta, df = df, qq = qq,
		description = "<Default>", theta.hat = theta.hat, xi.hat = xi.hat)
	return(res)
}

fit.mixture.link.kappa.only.mle <- function(y, m, J, p, Pi, extra.tx = null.tx,
	var.names = NULL, method = "imhof")
{
	if (length(m) == 1) m <- rep(m, length(y))
	if (length(p) == 1) p <- rep(p, length(y))
	
	Data <- list(y = y, m = m, n = length(y))
	qq <- 1
	
	phi.init <- log(1)
	
	theta.tx <- function(phi)
	{
		list(kappa = exp(phi))
	}
	
	loglik <- function(phi, Data)
	{
		theta <- theta.tx(phi)
		
		ll <- numeric(Data$n)
		for (i in 1:Data$n)
		{
			ll[i] <- d.mixture.link.one(Data$y[i], Data$m[i], p[i], Pi,
				theta$kappa, method = method, log = TRUE)
		}
		
		printf("kappa = %f, loglik = %f\n", theta$kappa, sum(ll))
		return(sum(ll))
	}
	
	start <- Sys.time()
	fit.out <- fit.mle(phi.init, loglik, theta.tx, extra.tx,
		Data, psi.names = var.names)
	
	fit.out$description <- paste0(
		sprintf("y[i] ~iid~ MixLink_%d(m[i], p, Pi, kappa),\n", J),
		"p and Pi known / given\n",
		sprintf("Using method: %s\n", method),
		sprintf("Elapsed time: %0.02f sec", as.numeric(Sys.time() - start, units = "secs"))
	)
	return(fit.out)
}

