printf <- function(msg, ...)
{
	cat(sprintf(msg, ...))
}

fprintf <- function(file, msg, ...)
{
	cat(sprintf(msg, ...), file = file)
}

my.numerical.format <- function(x, lower = 1e-4)
{
	idx1 <- which(abs(x) < lower)
	idx2 <- setdiff(1:length(x), idx1)
	y <- character(length(x))
	y[idx1] <- sprintf("%0.3E", x[idx1])
	y[idx2] <- sprintf("%0.4f", x[idx2])
	names(y) <- names(x)
	return(y)
}

print.debug <- function(x, debug = FALSE)
{
	if (debug) print(x)
}

normalize <- function(x) { x / sum(x) }

null.tx <- function(phi) { list() }

prob.simplex.tx <- function(x)
{
	## x is a k-1 dimensional real-valued vector
	## Transform x to k-dimensional probability simplex
	## Do one of two possible things, depending if k == 2
	k <- length(x) + 1

	if (k == 2) {
		fx <- c(plogis(x), 1-plogis(x))
	} else {
		N <- sum(plogis(x)) + plogis(sum(x))
		fx <- c(plogis(x), plogis(sum(x))) / N
	}

	return(fx)
}

integrate.trap <- function(f, xlim, n, ...)
{
	a <- xlim[1]
	b <- xlim[2]
	
	i <- seq(1, n)
	ff <- f(a + i/2 * (b-a)/n, ...)
	
	II <- (b - a) / (2*n) * sum(ff)
	return(II)
}


rpad.str <- function(x, len)
{
	L <- length(x)
	y <- character(L)
	pad <- len - nchar(x)

	for (i in 1:L)
	{
		s <- paste0(rep(" ", pad[i]), collapse="")
		y[i] <- sprintf("%s%s", x[i], s)
	}

	return(y)
}

## The following numerical derivative functions are not currently needed.
## We'll use numDeriv functions instead.

if (FALSE)
grad <- function(f, x, eps = 1e-4)
{
	k <- length(x)
	E <- diag(eps, k)	## Perturbations in each axis direction
  
	fx <- f(x)
	D <- numeric(k)
  
	for (j in 1:k)
	{
		D[j] <- f(x + E[j,]) - fx
	}
  
	return(D / eps)
}

if (FALSE)
hessian <- function(f, x, eps = 1e-4)
{
	k <- length(x)
	E <- diag(eps, k)	## Perturbations in each axis direction
  
	fx <- f(x)
	H <- matrix(NA, k, k)
  
	for (i in 1:k)
	{
		for (j in 1:k)
		{ 
			H[i,j] <- f(x + E[i,] + E[j,]) - f(x + E[i,]) - f(x + E[j,]) + fx
		}
	}
  
	return(H / eps^2)
}

if (FALSE)
jacobian <- function(f, x)
{
	k <- length(x)
	E <- diag(eps, k)	## Perturbation in each axis direction
  
	fx <- f(x)
	n <- length(fx)
	J <- matrix(NA, n, k)
  
	for (j in 1:k)  
	{
		fxe <- f(x + E[j,])
		J[,j] <- fxe - fx
	}
  
	return(J / eps)
}
